/**
 * @typedef {import('ipfs-message-port-protocol/src/data').EncodedError} EncodedError
 */
/**
 * @template X, T
 * @typedef {import('ipfs-message-port-protocol/src/data').Result<X, T>} Result
 */
/**
 * @template T
 * @typedef {import('ipfs-message-port-protocol/src/rpc').ProcedureNames<T>} ProcedureNames
 */
/**
 * @template T
 * @typedef {import('ipfs-message-port-protocol/src/rpc').Method<T>} Method
 */
/**
 * @template T
 * @typedef {import('ipfs-message-port-protocol/src/rpc').Namespace<T>} Namespace
 */
/**
 * @template T
 * @typedef {import('ipfs-message-port-protocol/src/rpc').ServiceQuery<T>} ServiceQuery
 */
/**
 * @template T
 * @typedef {import('ipfs-message-port-protocol/src/rpc').Return<T>} Return
 */
/**
 * @template T
 * @typedef {import('ipfs-message-port-protocol/src/rpc').RPCQuery<T>} RPCQuery
 */
/**
 * @template T
 * @typedef {import('ipfs-message-port-protocol/src/rpc').Inn<T>} Inn
 */
/**
 * @template T
 * @typedef {import('ipfs-message-port-protocol/src/rpc').Out<T>} Out
 */
/**
 * @template T
 * @typedef {object} QueryMessage
 * @property {'query'} type
 * @property {Namespace<T>} namespace
 * @property {Method<T>} method
 * @property {string} id
 * @property {Inn<T>} input
 */
/**
 * @typedef {object} AbortMessage
 * @property {'abort'} type
 * @property {string} id
 */
/**
 * @typedef {object} TransferOptions
 * @property {Set<Transferable>} [transfer]
 */
/**
 * @template O
 * @typedef {O & TransferOptions} QueryResult
 */
/**
 * @template T
 * @typedef {AbortMessage|QueryMessage<T>} Message
 */
/**
 * @template T, K
 * @typedef {import('ipfs-message-port-protocol/src/rpc').NamespacedQuery<T, K>} NamespacedQuery
 */
/**
 * Represents a client query received on the server.
 *
 * @template T
 * @extends {ServiceQuery<T>}
 */
export class Query<T> {
    /**
     * @param {Namespace<T>} namespace
     * @param {Method<T>} method
     * @param {Inn<T>} input
     */
    constructor(namespace: Namespace<T>, method: Method<T>, input: Inn<T>);
    /** @type {Return<any>} */
    result: Return<any>;
    succeed: (value: any) => void;
    fail: (reason?: any) => void;
    namespace: Namespace<T>;
    method: Method<T>;
    input: Inn<T>;
    abortController: AbortController;
    signal: AbortSignal;
    /**
     * Aborts this query if it is still pending.
     */
    abort(): void;
}
/**
 * @template T
 * @typedef {import('ipfs-message-port-protocol/src/rpc').MultiService<T>} MultiService
 */
/**
 * Server wraps `T` service and executes queries received from connected ports.
 *
 * @template T
 */
export class Server<T> {
    /**
     * @param {MultiService<T>} services
     */
    constructor(services: MultiService<T>);
    services: import("ipfs-message-port-protocol/src/rpc").MultiService<T>;
    /** @type {Record<string, Query<T>>} */
    queries: Record<string, Query<T>>;
    /**
     * @param {MessagePort} port
     */
    connect(port: MessagePort): void;
    /**
     * @param {MessagePort} port
     */
    disconnect(port: MessagePort): void;
    /**
     * Handles messages received from connected clients
     *
     * @param {MessageEvent} event
     * @returns {void}
     */
    handleEvent(event: MessageEvent): void;
    /**
     * Abort query for the given id.
     *
     * @param {string} id
     */
    abort(id: string): void;
    /**
     * Handles query received from the client.
     *
     * @param {string} id
     * @param {Query<T>} query
     * @param {MessagePort} port
     */
    handleQuery(id: string, query: Query<T>, port: MessagePort): Promise<void>;
    /**
     * @param {Query<T>} query
     * @returns {void}
     */
    run(query: Query<T>): void;
    /**
     * @param {RPCQuery<T>} data
     * @returns {Out<T>}
     */
    execute(data: RPCQuery<T>): Out<T>;
}
export class UnsupportedMessageError extends RangeError {
    /**
     * @param {MessageEvent} event
     */
    constructor(event: MessageEvent);
    event: MessageEvent<any>;
    get name(): string;
}
export const AbortError: {
    new (message?: string | undefined): {
        readonly name: string;
        message: string;
        stack?: string | undefined;
        cause?: Error | undefined;
    };
    new (message?: string | undefined, options?: ErrorOptions | undefined): {
        readonly name: string;
        message: string;
        stack?: string | undefined;
        cause?: Error | undefined;
    };
    captureStackTrace(targetObject: object, constructorOpt?: Function | undefined): void;
    prepareStackTrace?: ((err: Error, stackTraces: NodeJS.CallSite[]) => any) | undefined;
    stackTraceLimit: number;
};
export type EncodedError = import('ipfs-message-port-protocol/src/data').EncodedError;
export type Result<X, T> = import('ipfs-message-port-protocol/src/data').Result<X, T>;
export type ProcedureNames<T> = import('ipfs-message-port-protocol/src/rpc').ProcedureNames<T>;
export type Method<T> = import('ipfs-message-port-protocol/src/rpc').Method<T>;
export type Namespace<T> = import('ipfs-message-port-protocol/src/rpc').Namespace<T>;
export type ServiceQuery<T> = import('ipfs-message-port-protocol/src/rpc').ServiceQuery<T>;
export type Return<T> = import('ipfs-message-port-protocol/src/rpc').Return<T>;
export type RPCQuery<T> = import('ipfs-message-port-protocol/src/rpc').RPCQuery<T>;
export type Inn<T> = import('ipfs-message-port-protocol/src/rpc').Inn<T>;
export type Out<T> = import('ipfs-message-port-protocol/src/rpc').Out<T>;
export type QueryMessage<T> = {
    type: 'query';
    namespace: Namespace<T>;
    method: Method<T>;
    id: string;
    input: Inn<T>;
};
export type AbortMessage = {
    type: 'abort';
    id: string;
};
export type TransferOptions = {
    transfer?: Set<Transferable> | undefined;
};
export type QueryResult<O> = O & TransferOptions;
export type Message<T> = AbortMessage | QueryMessage<T>;
export type NamespacedQuery<T, K> = import('ipfs-message-port-protocol/src/rpc').NamespacedQuery<T, K>;
export type MultiService<T> = import('ipfs-message-port-protocol/src/rpc').MultiService<T>;
//# sourceMappingURL=server.d.ts.map